@@ -1,5 +1,5 @@
 """
-All exceptions thrown by service_identity.
+All exceptions and warnings thrown by ``service_identity``.
 
 Separated into an own package for nicer tracebacks, you should still import
 them from __init__.py.
@@ -7,35 +7,50 @@ them from __init__.py.
 
 from __future__ import absolute_import, division, print_function
 
-from characteristic import attributes
+import attr
 
 
-@attributes(["errors"])
+class SubjectAltNameWarning(DeprecationWarning):
+    """
+    Server Certificate does not contain a ``SubjectAltName``.
+
+    Hostname matching is performed on the ``CommonName`` which is deprecated.
+    """
+
+
+@attr.s
 class VerificationError(Exception):
     """
     Service identity verification failed.
     """
+    errors = attr.ib()
+
+    def __str__(self):
+        return self.__repr__()
 
 
-@attributes(["mismatched_id"])
+@attr.s
 class DNSMismatch(object):
     """
     Not matching DNSPattern could be found.
     """
+    mismatched_id = attr.ib()
 
 
-@attributes(["mismatched_id"])
+@attr.s
 class SRVMismatch(object):
     """
     Not matching SRVPattern could be found.
     """
+    mismatched_id = attr.ib()
 
 
-@attributes(["mismatched_id"])
+@attr.s
 class URIMismatch(object):
     """
     Not matching URIPattern could be found.
     """
+    mismatched_id = attr.ib()
 
 
 class CertificateError(Exception):
